<el-input-number
  v-model="state.{{{widget.widget_id}}}.value"
  :min="data.{{{widget.widget_id}}}.min"
  :max="data.{{{widget.widget_id}}}.max"
  :step="data.{{{widget.widget_id}}}.step"
  :size="data.{{{widget.widget_id}}}.size"
  :controls="data.{{{widget.widget_id}}}.controls"
  :debounce="data.{{{widget.widget_id}}}.debounce"
>
</el-input-number>
