# coding: utf-8

# fields names
BITMAP = 'bitmap'
COLS = 'cols'
DATA = 'data'
EXTERIOR = 'exterior'
FACES = 'faces'
INTERIOR = 'interior'
MULTICHANNEL_BITMAP = 'multichannelBitmap'
ORIGIN = 'origin'
POINTS = 'points'
ROWS = 'rows'
TYPE = 'type'
GEOMETRY_SHAPE = 'shape'
GEOMETRY_TYPE = 'geometryType'
ANY_SHAPE = 'any'

NODES = 'nodes'
EDGES = 'edges'
ENABLED = 'enabled'
LABEL = 'label'
COLOR = 'color'
TEMPLATE = 'template'
LOCATION = 'location'

X = "x"
Y = "y"
Z = "z"
POSITION = "position"
ROTATION = "rotation"
DIMENTIONS = "dimensions"


LABELER_LOGIN = 'labelerLogin'
UPDATED_AT = 'updatedAt'
CREATED_AT = 'createdAt'
ID = 'id'
CLASS_ID = 'classId'

INDICES = 'indices'
